import gurobipy as gp

from gurobipy import GRB


_GRB_BRUTE_ENV = gp.Env(empty=True)
_GRB_BRUTE_ENV.setParam("Threads", 1)
_GRB_BRUTE_ENV.setParam("OutputFlag", 0)  # <----   Gurobi is silenced!
_GRB_BRUTE_ENV.start()


def max_flow(B, ub, _gurobi_env=_GRB_BRUTE_ENV):
    """
    Finds the max-flow from vertex 0 (source) to vertex 1 (sink) given the
    incidence matrix B[v, e] of an undirected graph and upper-bounds `ub[e]`.
    """
    nv, ne = B.shape
    assert nv >= 2

    # SciPy csr @ Gurobi v10 bug with zero-sized matrix variables (Dec 2023)
    if ne == 0:
        return 0.0

    model = gp.Model(env=_gurobi_env)
    f = model.addMVar(shape=(ne), ub=ub)

    model.addConstr(B[2:, :] @ f == 0)
    model.setObjective(B[0, :] @ f, sense=GRB.MAXIMIZE)

    model.optimize()

    if model.status == GRB.OPTIMAL:
        return model.ObjVal
    else:
        return float("nan")
